@@ -10,7 +10,7 @@ from csnlp.util.funcs import cached_property, cache_clearer, np_random
 from csnlp.util.data import struct_symSX, dict2struct
 
 
-'''This dictates the order for operations related to dual variables.'''
+'''This tuple dictates the order for operations related to dual variables.'''
 _DUAL_VARIABLES_ORDER = ('g', 'h', 'h_lbx', 'h_ubx')
 
 
@@ -74,7 +74,7 @@ class Nlp:
         self._debug = NlpDebug()
         self._seed = seed
         self._np_random: Optional[npy.random.Generator] = None
-        self.remove_redundant_x_bounds = remove_redundant_x_bounds
+        self._remove_redundant_x_bounds = remove_redundant_x_bounds
 
     @property
     def unwrapped(self) -> 'Nlp':
@@ -89,11 +89,6 @@ class Nlp:
             self._np_random, _ = np_random(self._seed)
         return self._np_random
 
-    @np_random.setter
-    def np_random(self, value: npy.random.Generator) -> None:
-        '''Sets the nlp's random engine.'''
-        self._np_random = value
-
     @property
     def sym_type(self) -> Union[Type[cs.SX], Type[cs.MX]]:
         '''Gets the CasADi symbolic type used in this NLP scheme.'''
@@ -265,7 +260,7 @@ class Nlp:
         '''Gets the inequalities due to `lbx` and their multipliers. If
         `simplify_x_bounds=True`, it removes redundant entries, i.e., where
         `lbx == -inf`; otherwise, returns all lower bound constraints.'''
-        if not self.remove_redundant_x_bounds:
+        if not self._remove_redundant_x_bounds:
             return self._lbx[:, None] - self._x, self._lam_lbx
         idx = npy.where(self._lbx != -npy.inf)[0]
         if idx.size == 0:
@@ -277,7 +272,7 @@ class Nlp:
         '''Gets the inequalities due to `ubx` and their multipliers. If
         `simplify_x_bounds=True`, it removes redundant entries, i.e., where
         `ubx == +inf`; otherwise, returns all upper bound constraints.'''
-        if not self.remove_redundant_x_bounds:
+        if not self._remove_redundant_x_bounds:
             return self._x - self._ubx[:, None], self._lam_ubx
         idx = npy.where(self._ubx != npy.inf)[0]
         if idx.size == 0:
@@ -296,8 +291,8 @@ class Nlp:
         ----------
         all : bool, optional
             If `True`, all dual variables are included, even the multipliers
-            connected to redundant `lbx` or `ubx`. Otherwise, those are
-            removed. By default, `False`.
+            connected to redundant `lbx` or `ubx`. Otherwise, the redundant
+            ones are removed. By default, `False`.
 
         Returns
         ------
@@ -613,16 +608,14 @@ class Nlp:
         lam_g = sol['lam_g'][:self.ng, :]
         lam_h = sol['lam_g'][self.ng:, :]
 
-        if self._csXX == cs.SX:
-            vars_: struct_symSX = self.variables
+        vars_ = self.variables
+        if self._csXX is cs.SX:
             vals = vars_(sol['x'])
         else:
-            vars_: Dict[str, cs.MX] = self.variables
             vals = dict2struct({name: subsevalf(var, self._x, sol['x'])
                                 for name, var in vars_.items()})
-        old = cs.vertcat(
-            self._p, self._x, self._lam_g, self._lam_h,
-            self._lam_lbx, self._lam_ubx)
+        old = cs.vertcat(self._p, self._x, self._lam_g, self._lam_h,
+                         self._lam_lbx, self._lam_ubx)
         new = cs.vertcat(p, sol['x'], lam_g, lam_h, lam_lbx, lam_ubx)
         get_value = partial(subsevalf, old=old, new=new)
         solution = Solution(
