@@ -2,14 +2,14 @@ from typing import Dict, Tuple, Union, Optional
 import casadi as cs
 import numpy as np
 from csnlp.nlp import _DUAL_VARIABLES_ORDER
-from csnlp.wrappers.wrapper import Wrapper, NlpType
+from csnlp.wrappers.wrapper import Wrapper, Nlp
 from csnlp.solutions import Solution
 from csnlp.util.funcs import cached_property, cache_clearer
 from csnlp.util.data import cs2array, array2cs
-from csnlp.util.array import hojacobian
+from csnlp.util.array import hojacobian, hohessian
 
 
-class NlpSensitivity(Wrapper[NlpType]):
+class NlpSensitivity(Wrapper):
     '''
     Wraps an NLP to allow to perform numerical sensitivity analysis and compute
     its derivates. See [1] for nonlinear programming sensitivity analysis.
@@ -24,14 +24,14 @@ class NlpSensitivity(Wrapper[NlpType]):
 
     def __init__(
         self,
-        nlp: NlpType,
+        nlp: Nlp,
         include_barrier_term: bool = True
     ) -> None:
         '''Instantiates the wrapper for performing NLP sensitivities.
 
         Parameters
         ----------
-        nlp : NlpType
+        nlp : Nlp
             The NLP problem to be wrapped.
         include_barrier_term : bool, optional
             If `True`, includes in the KKT matrix a new symbolic variable that
@@ -201,8 +201,6 @@ class NlpSensitivity(Wrapper[NlpType]):
 
         Raises
         ------
-        ValueError
-            Raises if `expr` is not scalar.
         numpy.linalg.LinAlgError
             Raises if the KKT conditions lead to a singular matrix.
 
@@ -257,29 +255,33 @@ class NlpSensitivity(Wrapper[NlpType]):
                 array2cs(dydp),
                 array2cs(d2ydp2) if d2ydp2.ndim <= 2 else d2ydp2
             )
-        if not Z.is_scalar():
-            raise ValueError('Expression must be scalar.')
+        Zshape = Z.shape
+        Z = cs.vec(Z)
 
         p = self.nlp._p
         y, idx = self._y_idx
-        Zpp, Zp = (cs2array(o) for o in cs.hessian(Z, p))
-        Zyy, Zy = (cs2array(o) for o in cs.hessian(Z, y))
-        Zyp = cs2array(cs.jacobian(cs.jacobian(Z, y), p))
-        Zpy = cs2array(cs.jacobian(cs.jacobian(Z, p), y))
-        Zy = Zy[idx, :]
-        Zyy = Zyy[idx, :][:, idx]
-        Zyp = Zyp[idx, :]
-        Zpy = Zpy[:, idx]
+        Zpp, Zp = hohessian(Z, p)
+        Zyy, Zy = hohessian(Z, y)
+        Zyp = hohessian(Z, y, p)[0]
+        Zpy = hohessian(Z, p, y)[0]
+        Zpp = Zpp.squeeze((1, 3, 5))
+        Zp = Zp.squeeze((1, 3))
+        Zy = Zy[:, 0, idx, 0]
+        Zyy = Zyy.squeeze((1, 3, 5))[:, idx, :][..., idx]
+        Zyp = Zyp.squeeze((1, 3, 5))[:, idx, :]
+        Zpy = Zpy.squeeze((1, 3, 5))[..., idx]
 
-        dzdp = dydp.T @ Zy + Zp
-        d2zdp2 = (
-            Zpp +
-            (d2ydp2.transpose((1, 2, 0)) @ Zy).squeeze() +
-            dydp.T @ (Zyp + Zpy.T + Zyy.T @ dydp)
-        )
+        dzdp = Zy @ dydp + Zp
+        T1 = (d2ydp2.transpose((1, 2, 0)) @ Zy.T).transpose((2, 0, 1))
+        T2 = Zyy.transpose((0, 2, 1)) @ dydp + Zpy.transpose((0, 2, 1)) + Zyp
+        d2zdp2 = Zpp + T1 + dydp.T @ T2
+
+        np_ = self.nlp.np
+        dzdp = dzdp.reshape(Zshape + (np_,), order='F')
+        d2zdp2 = d2zdp2.reshape(Zshape + (np_, np_), order='F')
         if solution is not None:
             return solution.value(dzdp), solution.value(d2zdp2)
-        return array2cs(dzdp), array2cs(d2zdp2)
+        return dzdp, d2zdp2
 
     @cache_clearer(jacobians, hessians, hojacobians)
     def parameter(self, *args, **kwargs):
@@ -298,7 +300,7 @@ class NlpSensitivity(Wrapper[NlpType]):
         return self.nlp.minimize(*args, **kwargs)
 
     @property
-    def _y_idx(self) -> Tuple[Union[cs.SX, cs.MX], np.ndarray]:
+    def _y_idx(self) -> Tuple[Union[cs.SX, cs.MX], Union[slice, np.ndarray]]:
         '''Internal utility to return all the primal-dual variables and indices
         that are associated to non-redundant entries in the kkt conditions.'''
         if self.nlp._csXX is cs.SX:
